<footer class="footer">
    <div class="footer-right">
        <p>&copy; 2025 OIDC Plugin for MLflow</p>
    </div>
    <div class="footer-right">
        <a href="https://github.com/mlflow-oidc/" ><i class="fab fa-github"></i> Follow us on GitHub</a>
    </div>
</footer>
